@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.3 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2020, Mark Calabretta
+  WCSLIB 7.12 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2022, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -17,11 +17,9 @@
   You should have received a copy of the GNU Lesser General Public License
   along with WCSLIB.  If not, see http://www.gnu.org/licenses.
 
-  Direct correspondence concerning WCSLIB to mark@calabretta.id.au
-
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: ttab2.c,v 7.3.1.2 2020/08/17 11:20:48 mcalabre Exp mcalabre $
+  $Id: ttab2.c,v 7.12 2022/09/09 04:57:58 mcalabre Exp $
 *=============================================================================
 *
 * ttab2 tests the -TAB routines using PGPLOT for graphical display.  It
@@ -40,33 +38,25 @@
 
 #include <tab.h>
 
-#define K1 2
-#define K2 2
+// 2 x 2 lookup table parameters.
+const int M  = 2;
+const int K1 = 2;
+const int K2 = 2;
+const int map[] = {0, 1};
+const double crval[] = {0.0, 0.0};
 
 // Number of subdivisions on each side of the interpolation element.
-#define NP 128
+const int NP = 128;
+
 
 int main()
 
 {
-  // Set up a 2 x 2 lookup table.
-  const int M = 2;
-  const int K[] = {K1, K2};
-  const int map[] = {0, 1};
-  const double crval[] = {0.0, 0.0};
-
-  char text[80];
-  int i, j, k, l, l1, l2, l3, lstep, m, stat[NP*NP], status;
-  float array[NP][NP], clev[31], v0, v1, w;
-  const float scl = 2.0f/(NP-1);
-  float ltm[6];
-  double x[NP][NP][2], world[NP][NP][2];
-  struct tabprm tab;
-
   printf("Testing WCSLIB coordinate lookup table routines (ttab2.c)\n"
          "---------------------------------------------------------\n");
 
   // List status return messages.
+  int status;
   printf("\nList of tab status return values:\n");
   for (status = 1; status <= 5; status++) {
     printf("%4d: %s.\n", status, tab_errmsg[status]);
@@ -76,6 +66,7 @@ int main()
 
 
   // PGPLOT initialization.
+  char text[80];
   strcpy(text, "/xwindow");
   cpgbeg(0, text, 1, 1);
   cpgvstd();
@@ -84,10 +75,13 @@ int main()
   // The viewport is slightly oversized.
   cpgwnad(-0.65f, 1.65f, -0.65f, 1.65f);
 
-  for (l = 0; l <= 30; l++) {
+  float clev[31];
+  for (int l = 0; l <= 30; l++) {
     clev[l] = 0.2f*(l-10);
   }
 
+  float ltm[6];
+  const float scl = 2.0f / (NP-1);
   ltm[0] = -scl*(1.0f + (NP-1)/4.0f);
   ltm[1] =  scl;
   ltm[2] =  0.0f;
@@ -97,6 +91,8 @@ int main()
 
 
   // Set up the lookup table.
+  int K[] = {K1, K2};
+  struct tabprm tab;
   tab.flag = -1;
   if ((status = tabini(1, M, K, &tab))) {
     printf("tabini ERROR %d: %s.\n", status, tab_errmsg[status]);
@@ -104,19 +100,20 @@ int main()
   }
 
   tab.M = M;
-  for (m = 0; m < tab.M; m++) {
+  for (int m = 0; m < tab.M; m++) {
     tab.K[m] = K[m];
     tab.map[m] = map[m];
     tab.crval[m] = crval[m];
 
-    for (k = 0; k < tab.K[m]; k++) {
+    for (int k = 0; k < tab.K[m]; k++) {
       tab.index[m][k] = (double)k;
     }
   }
 
   // Subdivide the interpolation element.
-  for (i = 0; i < NP; i++) {
-    for (j = 0; j < NP; j++) {
+  double x[NP][NP][2];
+  for (int i = 0; i < NP; i++) {
+    for (int j = 0; j < NP; j++) {
       x[i][j][0] = j*(K1-1.0)*scl - 0.5 - crval[0];
       x[i][j][1] = i*(K2-1.0)*scl - 0.5 - crval[1];
     }
@@ -132,21 +129,23 @@ int main()
   tab.coord[1] = 0.0;
 
   // The second coordinate element varies in three of the corners.
-  for (l3 = 0; l3 <= 100; l3 += 20) {
+  for (int l3 = 0; l3 <= 100; l3 += 20) {
     // (k1,k2) = (1,1).
     tab.coord[7] = 0.01 * l3;
 
-    for (l2 = 0; l2 <= 100; l2 += 20) {
+    for (int l2 = 0; l2 <= 100; l2 += 20) {
       // (k1,k2) = (0,1).
       tab.coord[5] = 0.01 * l2;
 
       cpgpage();
-      for (l1 = 0; l1 <= 100; l1 += 2) {
+      for (int l1 = 0; l1 <= 100; l1 += 2) {
         // (k1,k2) = (1,0).
         tab.coord[3] = 0.01 * l1;
 
         // Compute coordinates within the interpolation element.
         tab.flag = 0;
+        int stat[NP*NP];
+        double world[NP][NP][2];
         if ((status = tabx2s(&tab, NP*NP, 2, (double *)x, (double *)world,
                              stat))) {
           printf("tabx2s ERROR %d: %s.\n", status, tab_errmsg[status]);
@@ -186,13 +185,15 @@ int main()
         cpgtext( 1.02f,  1.02f, text);
 
         cpgsci(1);
+        float v0, v1;
+
         // Contour labelling: bottom.
         v0 = world[0][0][1];
         v1 = world[0][NP-1][1];
         if (v0 != v1) {
-          lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
-          for (l = -200; l <= 300; l += lstep) {
-            w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
+          int lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
+          for (int l = -200; l <= 300; l += lstep) {
+            float w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
             if (w < -0.5 || w > 1.5) continue;
 
             sprintf(text, "%4.1f", l*0.01f);
@@ -204,9 +205,9 @@ int main()
         v0 = world[0][0][1];
         v1 = world[NP-1][0][1];
         if (v0 != v1) {
-          lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
-          for (l = -200; l <= 300; l += lstep) {
-            w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
+          int lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
+          for (int l = -200; l <= 300; l += lstep) {
+            float w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
             if (w < -0.5 || w > 1.5) continue;
 
             sprintf(text, "%4.1f", l*0.01f);
@@ -218,9 +219,9 @@ int main()
         v0 = world[0][NP-1][1];
         v1 = world[NP-1][NP-1][1];
         if (v0 != v1) {
-          lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
-          for (l = -200; l <= 300; l += lstep) {
-            w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
+          int lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
+          for (int l = -200; l <= 300; l += lstep) {
+            float w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
             if (w < -0.5 || w > 1.5) continue;
 
             sprintf(text, "%.1f", l*0.01f);
@@ -232,9 +233,9 @@ int main()
         v0 = world[NP-1][0][1];
         v1 = world[NP-1][NP-1][1];
         if (v0 != v1) {
-          lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
-          for (l = -200; l <= 300; l += lstep) {
-            w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
+          int lstep = (abs((int)((v1-v0)/0.2f)) < 10) ? 20 : 40;
+          for (int l = -200; l <= 300; l += lstep) {
+            float w = -0.5f + 2.0f * (l*0.01f - v0) / (v1 - v0);
             if (w < -0.5 || w > 1.5) continue;
 
             sprintf(text, "%4.1f", l*0.01f);
@@ -243,8 +244,9 @@ int main()
         }
 
         // Draw contours for the second coordinate element.
-        for (i = 0; i < NP; i++) {
-          for (j = 0; j < NP; j++) {
+        float array[NP][NP];
+        for (int i = 0; i < NP; i++) {
+          for (int j = 0; j < NP; j++) {
             array[i][j] = world[i][j][1];
           }
         }
