@@ -1,7 +1,7 @@
 *=======================================================================
 *
-* PGSBOX 7.4 - draw curvilinear coordinate axes for PGPLOT.
-* Copyright (C) 1997-2021, Mark Calabretta
+* PGSBOX 7.12 - draw curvilinear coordinate axes for PGPLOT.
+* Copyright (C) 1997-2022, Mark Calabretta
 *
 * This file is part of PGSBOX.
 *
@@ -20,7 +20,7 @@
 *
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: pgtest.f,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+* $Id: pgtest.f,v 7.12 2022/09/09 04:57:58 mcalabre Exp $
 *=======================================================================
       PROGRAM PGTEST
 *=======================================================================
@@ -44,9 +44,6 @@
       EXTERNAL FSCAN
       EXTERNAL PGCRFN
       EXTERNAL PGWCSL
-
-*     A portability fix - does '\' itself need to be escaped?
-      DATA ESCAPE /'\\'/
 *-----------------------------------------------------------------------
 *     Setup.
       NAXIS(1) = 512
@@ -57,7 +54,9 @@
       TRC(1) = NAXIS(1) + 0.5
       TRC(2) = NAXIS(2) + 0.5
 
-*      CALL PGBEG (0, '?', 1, 1)
+      ESCAPE = CHAR(92)
+
+*     CALL PGBEG (0, '?', 1, 1)
       call pgbeg (0, '/xw', 1, 1)
 
       CALL PGQINF ('TYPE', DEVTYP, J)
@@ -158,9 +157,11 @@
 
 *     Defer labelling.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 2D0, 0, 0D0,
-     :  0, 0D0, .FALSE., LNGVEL, 1, 1, 7, ' ', 0, NLDPRM, 256, IC,
-     :  CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., LNGVEL, 1, 1, 7, ' ', 0,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw fiducial grid lines in white and do labelling.
       CALL PGSCI (1)
@@ -168,9 +169,9 @@
       GCODE(2) = 2
       GRID1(1) = 180D0
       GRID2(1) = 0D0
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, C0, GCODE, 0D0, 1,
-     :  GRID1, 1, GRID2, .FALSE., LNGVEL, 1, 1, 7, ' ', 0, NLDPRM, 256,
-     :  IC, CACHE, IERR)
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, C0, GCODE, 0D0,
+     :  1, GRID1, 1, GRID2, .FALSE., LNGVEL, 1, 1, 7, ' ', 0,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGBOX ('BC', 0.0, 0, 'BC', 0.0, 0)
@@ -234,9 +235,11 @@
 *     Setting LABDEN = 9900 forces all logarithmic grid lines to be
 *     drawn.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 9900, C0, GCODE, 2D0, 0,
-     :  0D0, 0, 0D0, .FALSE., FSCAN, 1, 1, 7, ' ', 0, NLDPRM, 256, IC,
-     :  CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 9900, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., FSCAN, 1, 1, 7, ' ', 0,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGBOX ('BC', 0.0, 0, 'BC', 0.0, 0)
@@ -290,9 +293,11 @@
       GCODE(2) = 2
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, C0, GCODE, 2D0, 0, 0D0,
-     :  0, 0D0, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM, NLDPRM, 256,
-     :  IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGBOX ('BC', 0.0, 0, 'BC', 0.0, 0)
@@ -369,8 +374,10 @@
 *     Draw the celestial grid.  The grid density is set for each world
 *     coordinate by specifying LABDEN = 1224.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 1224, C0, GCODE, 0D0, 0,
-     :  0D0, 0, 0D0, .FALSE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 1224, C0, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
@@ -468,9 +475,11 @@
 
 *     Draw the celestial grid letting PGSBOX choose the increments.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, CI, GCODE, 0D0, 0, 0D0,
-     :  0, 0D0, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS, NLDPRM, 256,
-     :  IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, CI, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Set parameters to draw the native grid.
       STATUS = WCSPTD (WCS, WCS_CRVAL,  0D0, 1, 0)
@@ -486,8 +495,8 @@
       GRID2(2) =   90D0
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0, 2,
-     :  GRID1, 2, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0,
+     :  2, GRID1, 2, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
@@ -570,9 +579,11 @@
       TIKLEN = -2D0
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, TIKLEN, 0,
-     :  5D0, 0, 5D0, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS, NLDPRM,
-     :  256, IC, CACHE, IERR)
+      GRID1(0) = 5D0
+      GRID2(0) = 5D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, TIKLEN,
+     :  0, GRID1, 0, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Resetting the table index to zero causes information about the
 *     tick marks to be discarded.
@@ -586,8 +597,8 @@
 *     Draw the primary meridian and equator.
       GRID1(1) = 0D0
       GRID2(1) = 0D0
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0, 1,
-     :  GRID1, 1, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0,
+     :  1, GRID1, 1, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     At this point the axis-crossing table will have entries for the
@@ -602,8 +613,8 @@
       GRID1(3) = 270D0
       GRID2(1) = -90D0
       GRID2(2) =  90D0
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0, 3,
-     :  GRID1, 2, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0,
+     :  3, GRID1, 2, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the first set of 15 degree meridians and parallels in blue.
@@ -620,8 +631,8 @@
       GRID2(2) = -30D0
       GRID2(3) =  15D0
       GRID2(4) =  60D0
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0, 8,
-     :  GRID1, 4, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0,
+     :  8, GRID1, 4, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the second set of 15 degree meridians and parallels in red.
@@ -638,8 +649,8 @@
       GRID2(2) = -15D0
       GRID2(3) =  30D0
       GRID2(4) =  75D0
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0, 8,
-     :  GRID1, 4, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0,
+     :  8, GRID1, 4, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     The axis-crossing table has now accumulated information for all of
@@ -681,8 +692,8 @@
       GRID1(4) = 315D0
       GRID2(1) = -45D0
       GRID2(2) =  45D0
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2333, 0, CI, GCODE, 0D0, 4,
-     :  GRID1, 2, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2333, 0, CI, GCODE, 0D0,
+     :  4, GRID1, 2, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Native grid in green (delineates boundary).
@@ -696,8 +707,8 @@
       STATUS = WCSPTD (WCS, WCS_LONPOLE, 999D0, 0, 0)
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0, 2,
-     :  GRID1, 1, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, -1, 0, C0, GCODE, 0D0,
+     :  2, GRID1, 1, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
@@ -808,8 +819,10 @@
       CI(7) = 17
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2100, 0, CI, GCODE, 0D0, 0,
-     :  15D0, 0, 15D0, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      GRID1(0) = 15D0
+      GRID2(0) = 15D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2100, 0, CI, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Reset CRPIX previously modified by CYLFIX.
@@ -855,9 +868,11 @@
 
 *     Draw the celestial grid letting PGSBOX choose the increments.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 21, 0, CI, GCODE, 0D0, 0, 0D0,
-     :  0, 0D0, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS, NLDPRM, 256,
-     :  IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 21, 0, CI, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGSCI (1)
@@ -931,9 +946,11 @@
       CALL PGSCI (1)
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2121, 1212, C0, GCODE, 0D0, 0,
-     :  0D0, 0, 0D0, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS, NLDPRM,
-     :  256, IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2121, 1212, C0, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGBOX ('BC', 0.0, 0, 'BC', 0.0, 0)
@@ -1010,9 +1027,11 @@
       CALL PGSCI (1)
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2121, 1212, C0, GCODE, 0D0, 0,
-     :  0D0, 0, 0D0, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS, NLDPRM,
-     :  256, IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 2121, 1212, C0, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .TRUE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGBOX ('BC', 0.0, 0, 'BC', 0.0, 0)
@@ -1085,8 +1104,10 @@
       CALL PGSCI (1)
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 1212, C0, GCODE, 0D0, 0,
-     :  0D0, 0, 0D0, .FALSE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 1212, C0, GCODE, 0D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
@@ -1162,9 +1183,11 @@
 
 *     Set LABCTL = 21 to label the bottom and left edges only.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 21, 0, C0, GCODE, 2D0, 0, 0D0,
-     :  0, 0D0, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM, NLDPRM, 256,
-     :  IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 21, 0, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Information for labels on the right edge was stored in the
 *     crossing table on the first call to PGSBOX.  We now want to
@@ -1188,9 +1211,11 @@
 
 *     Set LABCTL = 12000 to label the right edge with the second
 *     coordinate without redrawing the grid lines.
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 12000, 0, C0, GCODE, 2D0, 0,
-     :  0D0, 0, 0D0, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM, NLDPRM,
-     :  256, IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 12000, 0, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     The alternative temperature scale in Fahrenheit is to be
 *     constructed with a new set of tick marks.
@@ -1209,9 +1234,11 @@
 *     Set LABCTL = 100 to label the top edge; Set IC = -1 to redetermine
 *     the coordinate extrema.
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 100, 0, C0, GCODE, 2D0, 0,
-     :  0D0, 0, 0D0, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM, NLDPRM,
-     :  256, IC, CACHE, IERR)
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 100, 0, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGCRFN, 8, 2, 4, FCODE, NLIPRM,
+     :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
       CALL PGBOX ('BC', 0.0, 0, 'BC', 0.0, 0)
@@ -1314,8 +1341,10 @@
       GCODE(2) = 1
 
       IC = -1
-      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, C0, GCODE, 2D0, 0,
-     :  0D0, 0, 0D0, .FALSE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
+      GRID1(0) = 0D0
+      GRID2(0) = 0D0
+      CALL PGSBOX (BLC, TRC, IDENTS, OPT, 0, 0, C0, GCODE, 2D0,
+     :  0, GRID1, 0, GRID2, .FALSE., PGWCSL, 1, WCSLEN, 1, NLCPRM, WCS,
      :  NLDPRM, 256, IC, CACHE, IERR)
 
 *     Draw the frame.
@@ -1327,7 +1356,7 @@
 
       STATUS = WCSFREE (WCS)
 
-      CALL PGASK (0)
+      CALL PGASK (.FALSE.)
       CALL PGEND ()
 
       END
