@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.7 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2021, Mark Calabretta
+  WCSLIB 7.12 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2022, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -19,7 +19,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: sundazel.c,v 7.7 2021/07/12 06:36:49 mcalabre Exp $
+  $Id: sundazel.c,v 7.12 2022/09/09 04:57:58 mcalabre Exp $
 *=============================================================================
 * Usage: sundazel [<option>] [<yyyymmdd>]
 *-----------------------------------------------------------------------------
@@ -95,7 +95,7 @@ char usage[] =
 "effective.\n";
 
 // To get the declaration of localtime_r() from time.h.
-#define _POSIX_C_SOURCE 1
+#define _POSIX_C_SOURCE 199506L
 
 #include <math.h>
 #include <time.h>
